package de.klausj.video.impl.gstreamer;

import java.awt.Component;
import java.io.File;
import java.nio.ByteBuffer;
import java.util.List;

import ips.media.MediaDecodingPlayer;
import ips.media.NativeMediaSystemException;
import ips.media.spi.NativeMediaSystem;
import ips.media.spi.VideoCaptureSourceInfo;
import ips.media.video.NativeWindowHandle;
import ipsk.beans.dyn.DynProperty;
import ipsk.beans.dyn.DynPropertyDescriptor;
import ipsk.text.Version;
import ipsk.util.LocalizableMessage;

public class GstVideoSystem implements NativeMediaSystem{
	final static boolean DEBUG = false;
	final static String NATIVE_LIB_NAME="libJavaNativeMediaGstLib";
	final static String DISPLAY_NAME="JNM Gstreamer";
	final static String NATIVE_LIB_LOCATION_KEY="ips.jnm.gstreamer.nativelib";
	
	static {
	    if (DEBUG) System.out.println("Loading native libs from:"+System.getenv("java.library.path"));
	    System.loadLibrary("jawt");
		Throwable loadError=null;
		String nativeLibLocation=null;
		try{
			nativeLibLocation=System.getProperty(NATIVE_LIB_LOCATION_KEY);	
		}catch(SecurityException se){
			//OK no debug mode, continue
		}
		
		if (nativeLibLocation!=null){
			if(DEBUG)System.out.println(DISPLAY_NAME+" library loading ... ");
			File dsLib=new File(nativeLibLocation);
			if(!dsLib.exists()){
				System.err.println(DISPLAY_NAME+" library not found!");
			}
			try{
				System.load(nativeLibLocation);
			}catch(UnsatisfiedLinkError ule){
				loadError=ule;
				System.err.println("Could not load "+DISPLAY_NAME);
				ule.printStackTrace();
				// this JNM implementation is not available
			}
		}else{
			try{
				System.loadLibrary(NATIVE_LIB_NAME);
			}catch(UnsatisfiedLinkError ule){
				loadError=ule;
//				// fail silently
//				// this JNM implementation is not available
				ule.printStackTrace();
			}
		}
		if (DEBUG){
			if(loadError==null){
				System.out.println(DISPLAY_NAME+" driver loaded");
			}
		}
		
	}

	private native void init();
	public GstVideoSystem(){
	    super();
	}
	
	public void initialize(){
	    init();
	}
	
	private ByteBuffer nativePointer;
	
	
	//public static native ByteBuffer init() throws VideoSystemException;
	
	public static GstMediaDecodingPlayer createDirectShowMediaDecodingPlayer(){
		return new GstMediaDecodingPlayer();
	}
	
	public static native void release() throws NativeMediaSystemException;
	
	
//	public static void main(String[] args){
////		try {
////			init();
////		} catch (VideoSystemException e) {
////			// TODO Auto-generated catch block
////			e.printStackTrace();
////		}
//		GstMediaDecodingPlayer dsmdp=createDirectShowMediaDecodingPlayer();
//		JVideoComponent jvc=new JVideoComponent();
//		dsmdp.addVideoTrackListener(jvc);
//		jvc.setPreferredSize(new Dimension(500,500));
//		JFrame f=new JFrame();
//		f.getContentPane().setLayout(new BorderLayout());
//		
//		f.getContentPane().add(jvc,BorderLayout.CENTER);
//		JPlayerControl pc=new JPlayerControl(dsmdp);
//		f.getContentPane().add(pc,BorderLayout.SOUTH);
//		
//		f.pack();
//		f.setVisible(true);
//		
////		File mediaFile=new File(args[0]);
//////		dsmdp.setMediaStreamSource(new FileStreamSource(mediaFile));
////		dsmdp.setNativeMediaFile(mediaFile);
//		try {
//			dsmdp.open();
//		} catch (NativeMediaSystemException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//		dsmdp.start();
////		//dsmdp.renderTestJ();
//		
//	}

	public LocalizableMessage getTitle() {
		
		return new LocalizableMessage("Gstreamer");
	}

	public LocalizableMessage getDescription() {
		
		return null;
	}

	public String getVendor() {
		
		return "gstreamer.org";
	}

	public Version getSpecificationVersion() {
		return new Version(new int[]{0,0,1});
	}

	public Version getImplementationVersion() {
		return new Version(new int[]{0,0,1});
	}

	public List<DynPropertyDescriptor> getDynamicPropertyDescriptors() {
		return null;
	}

	public void setDynamicProperty(DynProperty dynProperty) {
		
	}

	public DynProperty getDynamicProperty(String name) {
		return null;
	}

	public MediaDecodingPlayer createMediaDecodingPlayer() {
		return new GstMediaDecodingPlayer();
	}
	public void enumerateVideoCaptureSources() {
		
	}
	public List<VideoCaptureSourceInfo> getVideoCaptureSourceInfos() {
		return null;
	}
    
    public String[] getLinks() {
        
        return new String[]{"http://gstreamer.freedesktop.org"};
    }
    @Override
    public boolean supportsMediaDecodingPlayer() {
        return true;
    }
    @Override
    public boolean supportsVideoCapture() {
        return false;
    }
 
      
}
